#!/usr/bin/perl

###############################################################################
##OCSInventory Version NG Beta
##Copyleft Pascal DANEK 2005
##Web : http://ocsinventory.sourceforge.net
##
##This code is open source and may be copied and modified as long as the source
##code is always made freely available.
##Please refer to the General Public Licence http://www.gnu.org/ or Licence.txt
################################################################################

use strict;
package Ocs::Client;
use XML::Simple;
use LWP::UserAgent;
use Compress::Zlib;
use Net::IP qw(:PROC);
#Data in MB
use constant UNITE => 1024;
use constant LOGPATH => "/var/log/ocsinventory-client/ocsinv.log";
use constant VERSION =>9;#This is not the official version number. Useful to update.
######
#MAIN#
############################################################
my(	$req, $res, $ua, $Dversion, $AuthUpdate, @dmidecode, $inventory, %request, @lspci,
	%account, @config, $ServerName, $DeviceID, $old_deviceid, $YEAR, $MONTH, $DAY, $HOUR, $MIN, $SEC, 
	$xfree, @xconfig, $compress, $xml, $start, $end, $UnitID, $OrderID, $OrderYear, 
	$OrderOwner,$ProductID, $URI, %xmladm, %xmlconf, %options, $distro, $Iversion);

$start=time();
#Is there a user to watch to logs ? If not, we put it into the logs file
unless(defined($ENV{USER})){
	open STDOUT, ">>".LOGPATH;
}else{
	#We notify that an inventory had been manually generated
	open FILE, ">>".LOGPATH;
	print FILE localtime()." => Generated manually\n"; 
	close FILE;
}

#Error logs to STDOUT
open STDERR, ">>&STDOUT";

#Permissions checking
unless(-r "/dev/mem"){
	die localtime()." => You don't have enough rights to run this program\n";
}

#Configuration reading
my $xmlconf=XML::Simple::XMLin("/etc/ocsinventory-client/ocsinv.conf", SuppressEmpty => undef);
$ServerName = $xmlconf->{OCSFSERVER};
$DeviceID   = $xmlconf->{DEVICEID};
$Dversion   = $xmlconf->{DMIVERSION}?$xmlconf->{DMIVERSION}:1;
$Iversion   = $xmlconf->{IPDISCOVER_VERSION}?$xmlconf->{IPDISCOVER_VERSION}:1;
$AuthUpdate = 1 if $xmlconf->{UPDATE} eq '1';

#Account infos reading
my $i=0;
my $xmladm=XML::Simple::XMLin("/etc/ocsinventory-client/ocsinv.adm", ForceArray => [ 'ACCOUNTINFO' ]);
while($xmladm->{ACCOUNTINFO}->[$i]->{KEYNAME}){
	$account{$xmladm->{ACCOUNTINFO}->[$i]->{KEYNAME}} = $xmladm->{ACCOUNTINFO}->[$i]->{KEYVALUE};
	$i++;
}

#Execution parameters reading
for(@ARGV){
	if(/-local/) { $ServerName = 'local' }
	elsif(/-url=(\S+)/) { $ServerName = '$1' }
	elsif(/-update/){
		&_update('test');
		exit(0);
	}
	elsif(/-xml/){$xml=1}
	else{
		die print localtime()." => Unknown argument = $_\nArret...\n";
	}
		
}


#DeviceID checking
chomp(my $Device=`hostname`);

unless($DeviceID=~/$Device-(\d{4})-(\d{2})-(\d{2})-(\d{2})-(\d{2})-(\d{2})/){
	#We save the old deviceid in order to tell to the server that we changed
	$old_deviceid=$DeviceID;
	#Building of DeviceID
	($YEAR, $MONTH , $DAY, $HOUR, $MIN, $SEC)=(localtime (time))[5,4,3,2,1,0];
	$DeviceID=sprintf "%s-%02d-%02d-%02d-%02d-%02d-%02d",  $Device, ($YEAR+1900), ($MONTH+1), $DAY, $HOUR, $MIN, $SEC;
	
	#writing of ocs.conf
	$xmlconf{'DEVICEID'} = [ $DeviceID ];
	$xmlconf{'OCSFSERVER'} = [ $ServerName ];
	$xmlconf{'DMIVERSION'} = [ $Dversion ];
	$xmlconf{'IPDISCOVER_VERSION'} = [ $Iversion ];
	$xmlconf{'UPDATE'} = [ "1" ] if $AuthUpdate;
	my $xml = XML::Simple::XMLout( \%xmlconf, RootName => 'CONF' );
	open CONF, ">/etc/ocsinventory-client/ocsinv.conf";
	print CONF $xml;
	close CONF;
}

#Server default name
$ServerName="ocsinventory-ng" if($ServerName eq '');

#Proceed...
if(($ServerName=~/^local$/i) or $xml){
	&_inventory();
}else{
	#URI construction
	$URI="http://".$ServerName."/ocsinventory";
	#LWP object created
	$ua = LWP::UserAgent->new(keep_alive => 1);
	$ua->agent('OCS-NG_linux_client_v'.VERSION);
	#If Autoupdate is on
	if($AuthUpdate){die("Relaunch ocs agent. Quitting.\n\n") if &_update()};
	if(&_prolog()){
		&_inventory();
	}
	
}

#That's all
print localtime()." => Terminated... :-)\n";
$end=time();
print localtime()." => Execution time : ".($end - $start)." secs\n";

############
#SUBROUTINES
############
#
sub _hardware{
	my (	$OSVersion, $OSName, $OSComment, $DeviceType, $PhysicalMemory, 
		$SwapFileSize, $ip, @ip, $UsersLoggedOn, @users, $domain, $uptime,
		$UYEAR, $UMONTH , $UDAY, $UHOUR, $UMIN, $USEC, %processors );
	
	#Operating system informations
	chomp($OSName=`uname -s`);
	chomp($OSVersion=`uname -r`);
	chomp($OSComment=`uname -v`);
	chomp($DeviceType=`uname -m`);
	
	#Wich distro ?
	$distro = 'debian' if -e "/etc/debian_version";
	$distro = 'mandrake' if -e "/etc/mandrake-release";
	$distro = 'redhat' if -e "/etc/redhat-release";
	$distro = 'suse' if -e "/etc/SuSE-release";
	
	#Memory informations
	open MEMINFO, "/proc/meminfo";
	while(<MEMINFO>){
		if (/^memtotal\s*:\s*(\S+).*/i){$PhysicalMemory=$1;}
		if (/^swaptotal\s*:\s*(\S+).*/i){$SwapFileSize=$1;}
	}
	
	#Looking for ip addresses with ifconfig, except loopback
	for(`env LANGUAGE=us ifconfig`){#ifconfig in the path
		if(/^\s*inet add?r\s*:\s*(\S+).*/){($1=~/127.+/)?next:push @ip, $1};
	}
	$ip=join "/", @ip;
	
	#Logged on users
	for(`who`){
		/^(\S+)./;
		#
		push @users, $1 if(! &_already_in_array($1, @users));
	}
	#
	sub _already_in_array {
	  my $lookfor = shift;
	  my @array = @_;
	  foreach (@array){
	    return 1 if($lookfor eq $_);
	  }
	  return 0;
	}
	
	$UsersLoggedOn=join "/", @users;
	
	#Domain name 
	open RESOLV, "/etc/resolv.conf";
	while(<RESOLV>){
		if (/^(domain|search)\s+(.+)/){$domain=$2;}
	}
	#If no domain name and no workgroup name (samba), we send "WORKGROUP"
	#TODO:Check if samba is present and get the windows workgroup or NT domain name
	unless (defined($domain)){chomp($domain="WORKGROUP");}
	$domain=~s/^.\.(.)/$1/;
	
	#Uptime 
	open UPTIME, "/proc/uptime";
	$uptime=<UPTIME>;
	$uptime=~s/^(.+)\s+.+/$1/;
	close UPTIME;
		
	#Uptime conversion
	($UYEAR, $UMONTH , $UDAY, $UHOUR, $UMIN, $USEC)=(gmtime ($uptime))[5,4,3,2,1,0];
	#Write compliant with the ISO format
	$uptime=sprintf "%02d-%02d-%02d %02d:%02d:%02d", ($UYEAR-70), $UMONTH, ($UDAY-1), $UHOUR, $UMIN, $USEC;
		
	#Hostname
	chomp(my $DeviceName=`hostname`);
	
	#CPU
	open CPUINFO, "/proc/cpuinfo";
	my($nbproc, @processors);
	while(<CPUINFO>){
		if (/^processor\s*:/){(defined($nbproc))?($nbproc++):($nbproc=0);}
		if (/^model name\s*:\s*(.+)/i){$processors{Type}=$1;}
		if (/^cpu mhz\s*:\s*(\S+)\n/i){$processors{Speed}=$1;}
	}
		
	#Writing data
	$request{'CONTENT'}{'HARDWARE'}{'NAME'} = [ $DeviceName ];
	$request{'CONTENT'}{'HARDWARE'}{'WORKGROUP'} = [ $domain ];
	$request{'CONTENT'}{'HARDWARE'}{'OSNAME'} = [ $OSName."($distro)" ];
	$request{'CONTENT'}{'HARDWARE'}{'OSVERSION'} = [ $OSVersion ];
	$request{'CONTENT'}{'HARDWARE'}{'OSCOMMENTS'} = [ $OSComment ];
	$request{'CONTENT'}{'HARDWARE'}{'PROCESSORT'} = [ $processors{Type} ];
	$request{'CONTENT'}{'HARDWARE'}{'PROCESSORS'} = [ $processors{Speed} ];
	$request{'CONTENT'}{'HARDWARE'}{'PROCESSORN'} = [ $nbproc+1 ];
	$request{'CONTENT'}{'HARDWARE'}{'MEMORY'} = [ sprintf("%i",$PhysicalMemory/UNITE) ];
	$request{'CONTENT'}{'HARDWARE'}{'SWAP'} = [ sprintf("%i", $SwapFileSize/UNITE) ];
	$request{'CONTENT'}{'HARDWARE'}{'IPADDR'} = [ $ip ];
	$request{'CONTENT'}{'HARDWARE'}{'USERID'} = [ $UsersLoggedOn ];
	$request{'CONTENT'}{'HARDWARE'}{'TYPE'} = [ '8' ];
	$request{'CONTENT'}{'HARDWARE'}{'DESCRIPTION'} = [ "$DeviceType/$uptime" ];
	#$writer->dataElement("WINCOMPANY", ); Arf...
	#$writer->dataElement("WINOWNER", ); Arf...
	#$writer->dataElement("WINPRODID", ); Arf.
}
	
#Account Infos
sub _accountinfo{
	my $i=0;
	for(keys(%account)){
		$request{'CONTENT'}{'ACCOUNTINFO'}[$i]{'KEYNAME'} = [ $_ ]; 
		$request{'CONTENT'}{'ACCOUNTINFO'}[$i]{'KEYVALUE'} = [ $account{$_} ]; 
		$i++;
	}
}
	

sub _bios{
	#Parsing dmidecode output
	#Flag to 1 if found and flag tested to know if we are in the right section
	my(	$SystemSerial , $SystemModel, $SystemManufacturer, $BiosManufacturer, 
		$BiosVersion, $BiosDate, $YEAR, $MONTH, $DAY, $HOUR, $MIN, $SEC);
		
	my $flag=0;
	for(@dmidecode){
		if(/^dmi type 0,/i){$flag=1;}
		if((/^dmi type (\d+),/i) && ($flag)){($1!='0')?last:1;}
		if((/^vendor\s*:\s*(.+)/i) && ($flag)) { $BiosManufacturer = $1 }
		if((/^release date\s*:\s*(.+)/i) && ($flag)) { $BiosDate = $1 }
		if((/^version\s*:\s*(.+)/i) && ($flag)) { $BiosVersion = $1 }
	}
	
	$flag=0;
	for(@dmidecode){
		if(/^dmi type 1,/i){$flag=1;}
		if((/^dmi type (\d+),/i) && ($flag)){($1!='1')?last:1;}
		if((/^serial number\s*:\s*(.+)/i) && ($flag)) { $SystemSerial = $1 }
		if((/^product name\s*:\s*(.+)/i) && ($flag)) { $SystemModel = $1 }
		if((/^manufacturer\s*:\s*(.+)/i) && ($flag)) { $SystemManufacturer = $1 }
	}		
	
	#Writing of data
	$request{'CONTENT'}{'BIOS'}{'SMANUFACTURER'} = [ $SystemManufacturer ];
	$request{'CONTENT'}{'BIOS'}{'SMODEL'} = [ $SystemModel ];
	$request{'CONTENT'}{'BIOS'}{'SSN'} = [ $SystemSerial ];
	$request{'CONTENT'}{'BIOS'}{'BMANUFACTURER'} = [ $BiosManufacturer ];
	$request{'CONTENT'}{'BIOS'}{'BVERSION'} = [ $BiosVersion ];
	$request{'CONTENT'}{'BIOS'}{'BDATE'} = [ $BiosDate ];
}	
	

sub _accesslog{
	my ($YEAR, $MONTH , $DAY, $HOUR, $MIN, $SEC)=(localtime (time))[5,4,3,2,1,0];
	my $Date=sprintf "%02d-%02d-%02d %02d:%02d:%02d", ($YEAR+1900), ($MONTH+1), $DAY, $HOUR, $MIN, $SEC;
	
	$request{'CONTENT'}{'ACCESSLOG'}{'USERID'} = [ 'N/A' ];
	$request{'CONTENT'}{'ACCESSLOG'}{'LOGDATE'} = [ $Date ];
}

sub _memories{
	my (@memories, $n);
	my $flag=0;
	for(@dmidecode){
		if(/^dmi type 17,/i){$flag=1; (defined($n))?($n++):($n=0);}
		if((/^dmi type (\d+),/i) && ($flag)){($1!='17')?$flag=0:1;}
		#Using a hash array
		if((/^size\s*:\s*(\S+)/i) && ($flag)) {$request{'CONTENT'}{'MEMORIES'}[$n]{'CAPACITY'} = [ $1 ]}
		if((/^speed\s*:\s*(.+)/i) && ($flag)) {$request{'CONTENT'}{'MEMORIES'}[$n]{'SPEED'} = [ $1 ]}
		if((/^type\s*:\s*(.+)/i) && ($flag)) {$request{'CONTENT'}{'MEMORIES'}[$n]{'TYPE'} = [ $1 ]}
	}	
	#Same way for the other data wich need hash array structure
	for(0..$#memories){ $request{'CONTENT'}{'MEMORIES'}[$_]{'NUMSLOTS'} = [ $n+1 ]	}
}
	

sub _networks{
	#Same way that dmidecode but with ifconfig
	my (@network, @dhcp, $gateway, $n);

	for(`ifconfig`){
		if(/^([ea]th\S+)/i){
			if(defined($n)){
			  $n++; 
			}else{
			  $n=0;
			}
			$network[$n]{InterfaceName}=$1;
		}
		if(/lo/){last;}
		if(/(link|lien) encap:(\S+)/i){ $network[$n]{type} = $2 }
		if(/hwadd?r\s+(\S+)/i){ $network[$n]{ HardwareAddress} = $1 }
		if(/inet add?r:(\S+)/i){ $network[$n]{ip} = $1 }
		if(/\S*mas(k|que):(\S+)/i){ $network[$n]{netmask} = $2 }
		if(/^\s*UP/i){ $network[$n]{status} = 'Up' }
		if($network[$n]{netmask} and $network[$n]{ip} and !$network[$n]{ipsubnet}){
		  #To retrieve the subnet for this iface
		  my $binip = &ip_iptobin ($network[$n]{ip} ,4);
		  my $binmask = &ip_iptobin ($network[$n]{netmask} ,4);
		  my $subnet = $binip & $binmask;
		  $network[$n]{ ipsubnet } = &ip_bintoip($subnet,4) or die(Error());
		}
	}
	
	#Looking for dhcp server IP (using /var/lib/dhcp/dhclient.leases)
	$n=undef;
	if(-e "/var/lib/dhcp/dhclient.leases"){
		open DHCP, "/var/lib/dhcp/dhclient.leases";
		while(<DHCP>){
			if(/lease\s*{/i){
				(defined($n))?($n++):($n=0); 
			}
			#Interface name
			if(/interface\s+"(.+)"\s*;/){
				$dhcp[$n]{Interface}=$1;
			}
			#Server IP
			if(/option\s+dhcp-server-identifier\s+(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})\s*;/){
				$dhcp[$n]{IPdhcp}=$1;
			}
		}
		close(DHCP);
	
		#Join to the other informations
		for $n (0..$#network){
			for(0..$#dhcp){
				if ($dhcp[$_]{Interface} eq $network[$n]{InterfaceName}){
					$network[$n]{dhcp}=$dhcp[$_]{IPdhcp};
				}
			}
		}
	}
	#Looking for the gateway
	for(`route`){
		if(/^defau\S*\s+(\S+)/i){$gateway=$1;}
	}
	
	for(0..$#network){
		$request{'CONTENT'}{'NETWORKS'}[$_]{'DESCRIPTION'} = [ $network[$_]{InterfaceName} ];
		$request{'CONTENT'}{'NETWORKS'}[$_]{'TYPE'} = [ $network[$_]{type} ];
		$request{'CONTENT'}{'NETWORKS'}[$_]{'MACADDR'} = [ $network[$_]{HardwareAddress} ];
		$request{'CONTENT'}{'NETWORKS'}[$_]{'STATUS'} = [ $network[$_]{status} ];
		$request{'CONTENT'}{'NETWORKS'}[$_]{'IPADDRESS'} = [ $network[$_]{ip} ];
		$request{'CONTENT'}{'NETWORKS'}[$_]{'IPMASK'} = [ $network[$_]{netmask} ];
		$request{'CONTENT'}{'NETWORKS'}[$_]{'IPGATEWAY'} = [ $gateway ];
		$request{'CONTENT'}{'NETWORKS'}[$_]{'IPDHCP'} = [ $network[$_]{dhcp} ];
		$request{'CONTENT'}{'NETWORKS'}[$_]{'IPSUBNET'} = [ $network[$_]{ipsubnet} ];
	}
}
	

sub _drives{
	my $i=0;
	#Looking for mount points and disk space
	for(`df -TP`){
		if(/^(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\s+(\S+)\n/){
			$request{'CONTENT'}{'DRIVES'}[$i]{'TYPE'} = [ $1 ];
			$request{'CONTENT'}{'DRIVES'}[$i]{'FILESYSTEM'} = [ $2 ];
			$request{'CONTENT'}{'DRIVES'}[$i]{'TOTAL'} = [ sprintf("%i",($3/UNITE)) ];
			$request{'CONTENT'}{'DRIVES'}[$i]{'FREE'} = [ sprintf("%i",($5/UNITE)) ];
			$request{'CONTENT'}{'DRIVES'}[$i]{'VOLUMN'} = [ $7 ];
			$i++;
		}
	}
}
	

sub _storages{
	my(@disques, $model, $capacity, $media, $Manufacturer, $n, $flag, @rep, @scsi);
	#IDE disks 
	#Looking for disks name in /proc, 
	#...to access to info files of each drive
	@rep=glob "/proc/ide/hd*";
	$n=0;
	for(@rep){
		if (-e "$_/capacity"){
			open CAPACITY, "$_/capacity" ;
			chomp($capacity=<CAPACITY>);
		}
		
		open MODEL, "$_/model";
		open MEDIA, "$_/media";
		chomp($model=<MODEL>);
		
		chomp($media=<MEDIA>);
				
		#Media file
		($media=~/disk/i)?$media="Hard Disk":1;
		#We try to identify the vendor name
		if($model=~/(maxtor|western|sony|compaq|hewlett packard|ibm|seagate|toshiba|fujitsu|lg|samsung)/i){$Manufacturer=$1;}
		else{$Manufacturer="??";}
		
		#Filling the hash array structure
		if($capacity){$disques[$n]{capacity} = $capacity/UNITE;}
		else{$disques[$n]{capacity}=0;}
		$disques[$n]{model} = $model;
		$disques[$n]{media} = $media;
		$disques[$n]{manufacturer}=$Manufacturer;
		$disques[$n]{type}='IDE';
		$n++;
	}
	
	#SCSI drives(/proc/scsi)
	open SCSI, "/proc/scsi/scsi";
	while(<SCSI>){push @scsi, $_;}
	$flag=1;
	for(@scsi){
		if(/vendor:\s*(.+)\s*model:\s*(.+)\s+rev/i){
			($flag)?$flag=0:$n++;
			$disques[$n]{manufacturer}=$1;
			$disques[$n]{model}=$2;
			$disques[$n]{type}='SCSI';
			$disques[$n]{media}='N/A';
			$disques[$n]{capacity}='N/A';
			$disques[$n]{manufacturer}=~s/\s*$//;
			$disques[$n]{model}=~s/\s*$//;
		}
	}
		
	#Writing of data
	for (0..$#disques){
		$request{'CONTENT'}{'STORAGES'}[$_]{'MANUFACTURER'} = [ $disques[$_]{manufacturer} ];
		$request{'CONTENT'}{'STORAGES'}[$_]{'MODEL'} = [ $disques[$_]{model} ];
		$request{'CONTENT'}{'STORAGES'}[$_]{'DESCRIPTION'} = [ $disques[$_]{type} ];
		$request{'CONTENT'}{'STORAGES'}[$_]{'TYPE'} = [ $disques[$_]{media} ];
		$request{'CONTENT'}{'STORAGES'}[$_]{'DISKSIZE'} = [ $disques[$_]{capacity} ];
	}
}
	

sub _ports{
	my (@ports, $n);
	my $flag=0;
	for(@dmidecode){
		if(/^dmi type 8,/i){$flag=1; (defined($n))?($n++):($n=0);}
		if((/^dmi type (\d+),/i) && ($flag)){($1!='8')?$flag=0:1;}
		if((/^external reference designator\s*:\s*(.+)/i) && ($flag)) { $ports[$n]{name} = $1 }
		if((/^external connector type\s*:\s*(.+)/i) && ($flag)) { $ports[$n]{caption} = $1 }
		if((/^internal connector type\s*:\s*(.+)/i) && ($flag)) { $ports[$n]{description} = $1 }
		if((/^port type\s*:\s*(.+)/i) && ($flag)) { $ports[$n]{type} = $1 }
	}		
	
	for(0..$#ports){
		$request{'CONTENT'}{'PORTS'}[$_]{'TYPE'} = [ $ports[$_]{type} ];
		$request{'CONTENT'}{'PORTS'}[$_]{'NAME'} = [ $ports[$_]{name} ];
		$request{'CONTENT'}{'PORTS'}[$_]{'CAPTION'} = [ $ports[$_]{caption} ];
		$request{'CONTENT'}{'PORTS'}[$_]{'DESCRIPTION'} = [ $ports[$_]{description} ];
	}

}
	

sub _controllers{
	#The same way that dmidecode, but with lspci
	my $n=0;
	for(`lspci`){push @lspci, $_;}
	for(@lspci){
		/^\S+\s(.+):\s(.+)/;
		$request{'CONTENT'}{'CONTROLLERS'}[$n]{'NAME'} = [ $1 ];
		$request{'CONTENT'}{'CONTROLLERS'}[$n]{'CAPTION'} = [ $2 ];
		/\(rev\s*(\S+)\)\n$/i;
		$request{'CONTENT'}{'CONTROLLERS'}[$n]{'VERSION'} = [ $1 ];
		$n++;
	}	
}
	

sub _slots{	
	my ( $n );
	my $flag=0;
	for(@dmidecode){
		if(/^dmi type 9,/i){$flag=1; (defined($n))?($n++):($n=0);}
		if((/^dmi type (\d+),/i) && ($flag)){($1!='9')?$flag=0:1;}
		if((/^id\s*:\s*(.+)/i) && ($flag)) {$request{'CONTENT'}{'SLOTS'}[$n]{'DESIGNATION'} = [ $1 ] }
		if((/^type\s*:\s*(.+)/i) && ($flag)) {$request{'CONTENT'}{'SLOTS'}[$n]{'DESCRIPTION'} = [ $1 ] }
		if((/^designation\s*:\s*(.+)/i) && ($flag)) {$request{'CONTENT'}{'SLOTS'}[$n]{'NAME'} = [ $1 ] }
		if((/^current usage\s*:\s*(.+)/i) && ($flag)) {$request{'CONTENT'}{'SLOTS'}[$n]{'STATUS'} = [ $1 ] }
	}		
}
	

sub _monitors{
	my(@monitors, $n);
	#Looking for XFConfig
	#If we find, we keep it in $xfree
	-e "/etc/X11/XF86Config" and $xfree="/etc/X11/XF86Config";
	-e "/etc/X11/XF86Config-4" and $xfree="/etc/X11/XF86Config-4";
	-e "/etc/XF86Config" and $xfree="/etc/XF86Config";
	-e "/etc/Xconfig" and $xfree="/etc/Xconfig";
	-e "/usr/X11/lib/X11/XF86Config" and $xfree="/usr/X11/lib/X11/XF86Config";
	
	#If xfree config file found
	if($xfree){	
		open XCONFIG, $xfree;
		while(<XCONFIG>){
			push @xconfig, $_;
		}
			
		my $flag=0;
		for(@xconfig){
			if(/section\s+("monitor")/i){
				$flag=1;(defined($n))?($n++):($n=0);
			}
			if((/endsection/i) && $flag){$flag=0;}
			
			if((/identifier\s+"(.+)"/i) && ($flag)) { $monitors[$n]{name}=$1 }
			if((/vendorname\s+"(.+)"/i) && ($flag)) { $monitors[$n]{manufacturer}=$1 }
			if((/modelname\s+"(.+)"/i) && ($flag)) { $monitors[$n]{description}=$1 }
		}	
		
		for(0..$#monitors){
			$request{'CONTENT'}{'MONITORS'}[$_]{'MANUFACTURER'} = $monitors[$_]{manufacturer};
			$request{'CONTENT'}{'MONITORS'}[$_]{'CAPTION'} = $monitors[$_]{name};
			$request{'CONTENT'}{'MONITORS'}[$_]{'DESCRIPTION'} = $monitors[$_]{description};
		}
	}
}	
	

sub _videos{
	#Like controller, but more precise
	my( $ligne, $vendor, $name, $n );
	$n = 0;
	for $ligne (@lspci){
		if($ligne=~/graphics|vga|video/i){
			$ligne=~/\d+\s(.+):(.+)\n/;
			$request{'CONTENT'}{'VIDEOS'}[$n]{'CHIPSET'} = [ $1 ];
			$request{'CONTENT'}{'VIDEOS'}[$n]{'NAME'} = [ $2 ];
			$n++;
		}
	}
}
	

sub _sounds{
	#Like controller, but more precise
	my( $ligne, $vendor, $name, $n );
	$n=0;
	for $ligne (@lspci){
		if($ligne=~/audio/i){
			$ligne=~/\d+\s(.+):(.+)\n/;
			$request{'CONTENT'}{'SOUNDS'}[$n]{'MANUFACTURER'} = [ $1 ];
			$request{'CONTENT'}{'SOUNDS'}[$n]{'NAME'} = [ $2 ];
			$n++;
		}
	}
}
	

sub _inputs{
	my(@inputs, $n);
	
	my $flag=0;
	if($xfree){
		for(@xconfig){
			if(/section\s+("inputdevice")|
			section\s+("keyboard")|
			section\s+("pointer")/i){
				$flag=1;(defined($n))?($n++):($n=0);
			}
			if((/endsection/i) && $flag){$flag=0;}
			if((/identifier\s+"(.+)"/i) && ($flag)) { $request{'CONTENT'}{'INPUTS'}[$n]{'CAPTION'} = [ $1 ] }
			if((/driver\s+"(.+)"/i) && ($flag)) { $request{'CONTENT'}{'INPUTS'}[$n]{'TYPE'} = [ $1 ] }
			if((/option\s+"protocol"\s*"(.+)"/i) && ($flag)) { $request{'CONTENT'}{'INPUTS'}[$n]{'INTERFACE'} = [ $1 ] }
		}
	}		
}
	

sub _modems{
	my($ligne, $vendor, $name, $n, @modems);
	
	$n=0;
	for $ligne (@lspci){
		if($ligne=~/modem/i){
			$ligne=~/\d+\s(.+):(.+)\n/;
			$request{'CONTENT'}{'MODEMS'}[$n]{'NAME'} = [ $1 ];
			$request{'CONTENT'}{'MODEMS'}[$n]{'DESCRIPTION'} = [ $2 ];
			$n++;
		}
	}
}
	
	
sub _softwares{
	
	my ($n, @softwares, $packets, @liste, $ligne);
	#Working with debian and rpm packets management
	if($distro =~/debian/){
		$packets='deb';
	}elsif($distro=~/mandrake|redhat|suse/){
		$packets='rpm';
	}
	#DEBIAN case
	if($packets && ($packets eq 'deb')){
		@liste=`dpkg -l`;
		$n=0;
		for $ligne(@liste){
			if ($ligne=~/^[uirph]/){
				$ligne=~/^(\w+)\s+(\S+)\s+(\S+)\s+(.*)/;
				$softwares[$n]{status}=$1;
				$softwares[$n]{name}=$2;
				$softwares[$n]{version}=$3;
				$softwares[$n]{description}=$4;
				$n++;
			}
		}
	}
	
	#RPM case
	if($packets && ($packets eq 'rpm')){
		@liste=`rpm -qa --queryformat "%{NAME} %{VERSION} %{SUMMARY}\n"`;
		
		$n=0;
		for $ligne(@liste){
				$ligne=~/^(\S+)\s+(\S+)\s+(.*)/;
				$softwares[$n]{status}='';
				$softwares[$n]{name}=$1;
				$softwares[$n]{version}=$2;
				$softwares[$n]{description}=$3;
				$n++;
		}
	}
	
	for(0..$#softwares){
		$request{'CONTENT'}{'SOFTWARES'}[$_]{'NAME'} = [ $softwares[$_]{name} ];
		$request{'CONTENT'}{'SOFTWARES'}[$_]{'VERSION'} = [ $softwares[$_]{version} ];
		$request{'CONTENT'}{'SOFTWARES'}[$_]{'COMMENTS'} = [ "$softwares[$_]{description}($softwares[$_]{status})" ];
	}
}

sub _generate{
	my (@account, $err, $ptmontage, $values, $i, $content);
	
	#If $OCSFServer is local, we know that the inventory is generated on the local machine and that we have to ask to user where he wants to put it.
	if(($ServerName eq 'local') or $xml){
		$err=1;
		my $n;
		while($err){
			print "\n\n\tChoose your target directory ? :  ";
			chomp($ptmontage=<STDIN>);
			#Checking existence and permissions of the destination directory
			if(!(-e $ptmontage)){
				print "The directory do not exist .\n";
			}elsif(!(-w $ptmontage)){
				print "The mount point is not writeable.\n"
			#Generate...
			}else{
				$err=0; 
				open XML, ">$ptmontage/$DeviceID.ocs" or die localtime()." => Cannot create file : $!\n";
				if($xml){print XML $inventory;
				}else{
					$inventory=Compress::Zlib::compress( $inventory ) or die localtime()." => Compression error\n";
					binmode XML;
					print XML $inventory
				}
				close XML;
			}
			$n++;
			die("Abort.\n") if $n>=3;
		}
		print localtime()." => Inventory generated...\n";
	}else{
		#Sending inventory to web server
		#Request construction
		undef $req;
		$req = HTTP::Request->new(POST => $URI);
		$req->header('Pragma' => 'no-cache', 'content-type' => 'application/x-compress', 'Connection' => 'keep-alive');
		$inventory=Compress::Zlib::compress( $inventory ) or die localtime()." => Compression error\n";
		$req->content($inventory);
		#Sending
		my $res = $ua->request($req);
		
		#Checking response
		if($res->is_success){
			print localtime()." => Transmission...done.\n";
		}else{
			print localtime()." => Cannot transmit inventory ->".$res->status_line, "\n";
			exit(1);
		}
		#If we receive account informations, we write it to the config file
		$content = _uncompress($res->content)  or die localtime()." => Cannot read the server message (code $content)\n";
		$xml=XML::Simple::XMLin( $content, ForceArray => ['ACCOUNTINFO', 'KEYNAME', 'KEYVALUE'], SuppressEmpty => undef);
		if($xml->{RESPONSE}=~/^no_account_update/i){
			print localtime()." => Account infos up to date\n";
								
		}elsif($xml->{RESPONSE}=~/^account_update/i){
			delete $xml->{RESPONSE};
			$xml=XML::Simple::XMLout( $xml, RootName => 'ADM', NoSort => 1 );
			print localtime()." => Updating Account infos\n";
			open ADM, ">/etc/ocsinventory-client/ocsinv.adm";
			print ADM $xml;
			close ADM;
		}else{
			print localtime()." => Problem with account infos : $content\n";
		}
	}
}

sub _inventory{
	#dmidecode is embedded in order to read the smbios headers
	#Getting the output
	die localtime()." => Cannot find dmidecode\n" unless -e "/usr/sbin/dmidecode";
	@dmidecode=`/usr/sbin/dmidecode`;
	for (@dmidecode){s/^\s*(\S*)/$1/;}
	#Begin of xml
	$request{'QUERY'} = ['INVENTORY'];
	#Running inventory
	#Writing of DeviceID
	$request{'CONTENT'}{'DEVICEID'} = [ $DeviceID ];
	$request{'CONTENT'}{'OLD_DEVICEID'} = [ $old_deviceid ] if $old_deviceid;
		
	&_hardware;
	&_accountinfo;
	&_bios;
	&_accesslog;
	&_memories;
	&_networks;
	&_drives;
	&_storages;
	&_ports;
	&_controllers;
	&_slots;
	&_monitors;
	&_videos;
	&_sounds;
	&_inputs;
	&_modems;
	&_softwares;
	&_options;
	$inventory = XMLout( \%request, RootName => 'REQUEST', XMLDecl => '<?xml version="1.0" encoding="ISO-8859-1"?>', NoSort => 1, SuppressEmpty => undef );
	#Create inventory
	&_generate;
}

#We ask to server if we have to send an inventory
sub _prolog{
	my ($message, $xml, $resp, $content);
	#Generation of xml message
	$request{'QUERY'}=['PROLOG'];
	$request{'DEVICEID'}=[$DeviceID];
	$message=XMLout( \%request, RootName => 'REQUEST', XMLDecl => '<?xml version="1.0" encoding="ISO-8859-1"?>',
	                 NoSort => 1, SuppressEmpty => undef );
	#####
	#HTTP
	#####
	undef $req;
	$req = HTTP::Request->new(POST => $URI);
	#To avoid proxys
	$req->header('Pragma' => 'no-cache', 'Content-type', 'application/x-compress');
	$message= Compress::Zlib::compress( $message )  or die localtime()." => Probleme de compression(prolog)\n";
	$req->content($message);
	$res = $ua->request($req);
	
	#Checking if connected
	unless($res->is_success) {
		die localtime()." => Cannot establish communication : ".$res->status_line, "\n";
	}
	#stop or send in the http's body
	$content = _uncompress($res->content)  or die localtime()." => Deflating problem (prolog, code= $content )\n";
	$xml=XML::Simple::XMLin( $content, ForceArray => ['OPTION'] );
	#Options management (ipdiscover only is implemented)
	my $i=0;
	while($xml->{OPTION}->[$i]->{NAME}){
	  if( $xml->{OPTION}->[$i]->{NAME} =~/ipdiscover/oi){
	    $options{'ipdiscover'} = $xml->{OPTION}->[$i]->{PARAM};
	  }
	  $i++;
	}
	if($xml->{RESPONSE} eq 'STOP'){
		print localtime()." => Inventory not generated, command by server...\n";
		return(0);
	}elsif($xml->{RESPONSE} eq 'SEND'){
		return(1);
	}else{
		die localtime()." => Server response unreadable : ".$content."\n";
	}
}

sub _uncompress{
	my($status, $data, $d);
	#Uncompress request
	($d, $status) = Compress::Zlib::inflateInit();
	($data, $status) = $d->inflate($_[0]);
	return ($data);
}

#Update or not
sub _update{
	
	my (@resp, $ASversion, $ISversion, $DSversion, @conf, $res, $old, $message, $xml, $uua, $ureq, $uURI, $content, $ipflag, $dmiflag);
	my $update=-1;
		
	#Sending our current version
	#Generation of xml message
	$request{'QUERY'}      = [ 'UPDATE' ]; 
	$request{'PLATFORM'}   = [ 'LINUX' ]; 
	$request{'AGENT'}      = [ VERSION ]; 
	unless($_[0] eq 'test'){
	  $request{'DMI'}        = [ $Dversion ];
	  $request{'IPDISCOVER'} = [ $Iversion ];
	}
	
	$message = XMLout( \%request, RootName => 'REQUEST', XMLDecl => '<?xml version="1.0" encoding="ISO-8859-1"?>',
	                  NoSort => 1, SuppressEmpty => undef );
	
	#sending
	if($_[0] eq 'test'){
	
		$uURI = "http://".$ServerName."/ocsinventory";
		$uua  = LWP::UserAgent->new();
		$ureq = HTTP::Request->new(POST => $uURI);
		
		$ureq->header('Pragma' => 'no-cache', 'Content-type' => 'application/x-compress');
		$message=Compress::Zlib::compress( $message )  or die localtime()." => Deflation problem (update)\n";
		$ureq->content($message);
		$res = $uua->request($ureq);
	}else{
		$req = HTTP::Request->new(POST => $URI);
		$req->header('Pragma' => 'no-cache', 'Content-type' => 'application/x-compress', 'Connection' => 'Keep-alive');
		$message=Compress::Zlib::compress( $message )  or die localtime()." => Inflating problem (update)\n";
		$req->content($message);
		$res = $ua->request($req);
	}
	
	#If connection problems
	unless($res->is_success) {
		die localtime()." => Update failed : ".$res->status_line, "\n";
	}
	#Put the response in xml format
	$content = _uncompress($res->content)  or die localtime()." => Inflating problem (update)\n";
	$xml=XML::Simple::XMLin( $content , SuppressEmpty => undef);
	
	#If we are currently testing an update, the server MUST answer update=0
	if(($_[0] eq 'test') and ($xml->{RESPONSE} eq 'UPDATE')){exit(1)}
	
	#The server tells us what files to ask
	if($xml->{RESPONSE} eq 'UPDATE'){
		$update=1;
	}elsif($xml->{RESPONSE} eq 'NO_UPDATE'){
		$update=0;
	}else{
		print localtime()." => Update response not readable : ".$content."\n";
		return;
	}
	#Update or no
	if($update){
		#Wich versions ?
		if(($xml->{AGENT})){$ASversion=$xml->{AGENT}}
		if(($xml->{DMI})){$DSversion=$xml->{DMI}}
		if(($xml->{IPDISCOVER})){$ISversion=$xml->{IPDISCOVER}}
		#DMI AGENT
		if(($DSversion)){
			undef $message;
			print localtime()." => updating dmi reader...to the version $DSversion\n";
			#Same way, we ask the file
			$URI = "http://".$ServerName."/ocsinventory/update/linux/dmidecode/$DSversion/";
			$req = HTTP::Request->new(GET => $URI);
			$res = $ua->request($req) or die localtime()." => Cannot get dmidecode\n = $!";
			unless($res->is_success){
				die localtime()." => Update problem with dmidecode\n";
			}
			$content = _uncompress($res->content)  or die localtime()." => Inflating problem (update)\n";
			########
			#UPDATE#
			#############################
			#Put the binary content in memory
			open DMI, "/usr/sbin/dmidecode" 
				or die "Cannot update dmidecode : $!\n";
			undef $old;
			while(<DMI>){
				$old.=$_;
			}
			open DMI, ">/usr/sbin/dmidecode";
			#Writing the new data
			print DMI $content;
			close DMI;
			#Testing the return value of dmidecode
			if(system("/usr/sbin/dmidecode > /dev/null")){
				open DMI, ">/usr/sbin/dmidecode"
					or die "Cannot go back to the previous version : $!\n";
				print DMI $old;
				close DMI;
				print localtime()." => Problem with dmi update. Go back to the previous version\n";
			}else{
				$dmiflag = 1;
				print localtime()." => Updating dmi\n";
			}
		}
		#IPDISCOVER AGENT
		if($ISversion){
			undef $URI;
			undef $old;
			undef $message;
			
			print localtime()." => updating ipdiscover...to the version $ISversion\n";
			#Same way, we ask the file
			$URI = "http://".$ServerName."/ocsinventory/update/linux/ipdiscover/$ISversion/";
			$req = HTTP::Request->new(GET => $URI);
			$res = $ua->request($req) or die localtime()." => Cannot get ipdiscover\n = $!";
			unless($res->is_success){
				die localtime()." => Update problem with ipdiscover\n";
			}
			$content = _uncompress($res->content)  or die localtime()." => Inflating problem (update)\n";
			########
			#UPDATE#
			#############################
			#Put the binary content in memory
			open IPDISCOVER, "/usr/sbin/ipdiscover" 
				or die "Cannot update ipdiscover : $!\n";
			undef $old;
			while(<IPDISCOVER>){
				$old.=$_;
			}
			open IPDISCOVER, ">/usr/sbin/ipdiscover";
			#Writing the new data
			print IPDISCOVER $content;
			close IPDISCOVER;
			#Testing the return value of ipdiscover
			if(system("/usr/sbin/ipdiscover > /dev/null")){
				open IPDISCOVER, ">/usr/sbin/ipdiscover"
					or die "Cannot go back to the previous version : $!\n";
				print IPDISCOVER $old;
				close IPDISCOVER;
				print localtime()." => Problem with ipdiscover update. Go back to the previous version\n";
			}else{
				$ipflag = 1;
				print localtime()." => Updating ipdiscover\n";
			}
		}
		#Writing new versions to .conf
		if($ipflag or $dmiflag){
		  #writing ocsinv.conf
		  my %xmlconf;
		  $xmlconf{'DEVICEID'}   = [ $DeviceID ];
		  $xmlconf{'OCSFSERVER'} = [ $ServerName ];
		  $xmlconf{'DMIVERSION'} = $dmiflag?[ $DSversion ]:[ $Dversion ];
		  $xmlconf{'IPDISCOVER_VERSION'} = $ipflag?[ $ISversion ]:[ $Iversion ];
		  $xmlconf{'UPDATE'}     = [ "1" ] if $AuthUpdate;
		  my $xml = XML::Simple::XMLout( \%xmlconf, RootName => 'CONF' );
		  open CONF, ">/etc/ocsinventory-client/ocsinv.conf";
		  print CONF $xml;
		  close CONF;
		}
		
		#OCS AGENT
		if(($ASversion)){
			undef $message;
			undef $URI;
			undef $old;
			print localtime()." => Updating agent...to the version $ASversion\n";
			#We ask the file
			$URI = "http://".$ServerName."/ocsinventory/update/linux/agent/$ASversion/";
			$req = HTTP::Request->new(GET => $URI);
			$res = $ua->request($req) or die localtime()." => Cannot get update\n = $!";
			unless($res->is_success){
				die localtime()." => Cannot get update\n";
			}
			$content = _uncompress($res->content)  or die localtime()." => Inflation problem(update)\n";
			########
			#UPDATE#
			#############################
			#We keep the old content in memory
			open AGENT, $0;
			while(<AGENT>){
				$old.=$_;
			}
			open AGENT, ">$0";
			#We write the content of the sending file
			print AGENT $content;
			close AGENT;
			#We run the script with -update parameter, to make only an update verification
			#We MUST be able to go back to previous version
			#The script must send 'ok'
			unless(`$0 -update` eq 'ok'){
				#If not, we restore the previous version
				open AGENT, ">$0";
				print AGENT $old;
				close AGENT;
				print localtime()." => Update problem. Go back to the previous version\n";
			}else{				
				print localtime()." => Agent updated\n";
				return(1);
			}
		}
	}else{
		#If the script is running with the -update parameter, we send 'ok'
		if($_[0] eq 'test'){
			if($update==0){print 'ok'}
		}else{
			print localtime()." => No update\n";
		}
		
	} 
	0;
}

sub _options{
   	if($options{'ipdiscover'}){
	  my $i=0;
	  my ($iface,$ipdiscover);
	  #We get the iface name for the related subnet
	  while($request{'CONTENT'}{'NETWORKS'}[$i]{'DESCRIPTION'}[0]){
	   if($request{'CONTENT'}{'NETWORKS'}[$i]{'IPSUBNET'}[0] eq $options{'ipdiscover'}){
	      $iface = $request{'CONTENT'}{'NETWORKS'}[$i]{'DESCRIPTION'}[0];
	      last;
	    }
	  $i++;
	  }
	return unless($iface);
	#and we launch the ipdiscover
	$ipdiscover = `/usr/sbin/ipdiscover $iface`;
		if($ipdiscover){
		 	$request{'CONTENT'}{'IPDISCOVER'} = XML::Simple::XMLin( $ipdiscover, ForceArray => 'H');
		}
	}
}
